# -*- coding: utf-8 -*-
from Handlers import BaseHandler
import pexpect
import os
import time
#-------------------------------------------------------------------------------
#
#-------------------------------------------------------------------------------
class Samba(BaseHandler.BaseHandler):
	#=======================================================
	# 
	#=======================================================
	def __init__(self):
		self.name = 'Samba'
		self.config = "/etc/samba/smb.conf"
		self.username_map = "/etc/samba/smbusers"
		self.username = 'smbuser'
		self.path = '/mnt/usbssd/%s' % self.username
		self.make_directory()

	#=======================================================
	# 
	#=======================================================
	def Get(self, request):

		status = 200
		result = {'samba':'', 'capacity':{}}
		result['samba'] = self.readConfig()
		result['capacity'] = self.ssd_size()
		return result, status

	#=======================================================
	# 
	#=======================================================
	def Put(self, request):

		conf = request.json['samba']
		status = self.writeConfig(conf)
		if status == 200:
			status = self.smbpasswd(conf['password'])
		return status

	#=======================================================
	# 
	#=======================================================
	def Post(self, request):

		status = self.ssd_format()
		if status == 200:
			conf = self.readConfig()
			self.make_directory()
		return status

	#=======================================================
	# 
	#=======================================================
	def readConfig(self):

		conf = {'directory':'', 'userid':'' }

		fd = open(self.config, "r")
		for line in fd:
			if line.startswith('[') and not('[global]' in line or '[print' in line):
				conf['directory'] = line.strip('[]\n')
				break
		fd.close()
		fd = open(self.username_map, "r")
		for line in fd:
			if self.username in line:
				conf['userid'] =  line.split('=')[1].strip('\n').strip('"')
				break
		fd.close()
		return conf

	#=======================================================
	# 
	#=======================================================
	def ssd_size(self):
		size = {}
		key = ['all', 'used', 'available', 'percentage']
		try:
			result = self.check_output('/bin/df -k /mnt/usbssd' , shell=True).split('\n')[1].split()
			result.pop(0)
			for i in range(4):
				size[key[i]] = result[i]
		except Exception as e:
			print (e)
			self.exception_log(e, 'ssd size')
		return size
	#=======================================================
	# 
	#=======================================================
	def writeConfig(self, conf):
		status = 200
		write_lines = []
		try:
			fd = open(self.config, "r+")
			for line in fd:
				if line.startswith('[') and not('[global]' in line or '[print' in line):
					write_lines.append( '[%s]\n' % conf['directory'] )
				else:
					write_lines.append( line )
			fd.seek(0)
			fd.write(''.join(write_lines))
			fd.truncate()
			fd.close()
			fd = open(self.username_map, "w")
			fd.write('%s="%s"' % (self.username, conf['userid']))
			fd.close()
			self.service('smbd', 'restart')
		except Exception as e:
			status = 400
			print (e)
			self.exception_log(e, 'write config')

		return status

	#=======================================================
	# 
	#=======================================================
	def smbpasswd(self, passwd):

		status = 200
		try:
			c = pexpect.spawn('/usr/bin/smbpasswd %s' % self.username)
			c.expect("New SMB password:")
			c.sendline(passwd)
			c.expect("Retype new SMB password:")
			c.sendline(passwd)
			c.readline()
			c.close

		except Exception as e:
			status = 409
			print (e)
			self.exception_log(e, 'smbpasswd')

		return status

	#=======================================================
	# 
	#=======================================================
	def make_directory(self):
		if not os.path.exists(self.path):
			os.mkdir(self.path)
		self.call('/bin/chown %s %s' % (self.username, self.path), shell=True)
		self.call('/bin/chgrp %s %s' % (self.username, self.path), shell=True)

	#=======================================================
	# 
	#=======================================================
	def ssd_format(self):
		status = 200
		try:
			lines = self.check_output('mount', shell=True)
			if lines:
				for line in lines.split('\n'):
					
					if '/mnt/usbssd' in line:
						device = line.split(' ')[0]
						break
				self.service('smbd', 'stop')
				if self.call('umount /mnt/usbssd', shell=True):
					self.log('ssd format: umount failed.')
					raise
				if self.call('echo -e "\no\nn\np\n1\n\n\nw" | fdisk %s' % device[:-1], shell=True):
					self.log('ssd format: fdisk failed.')
					raise
				time.sleep(0.5)
				lines = self.check_output('mount', shell=True)
				if '/mnt/usbssd' in lines:
					self.call('umount /mnt/usbssd', shell=True)

				if  self.call('mkfs.ext4 %s' % device, shell=True):
					self.log('ssd format: mkfs failed.')
					raise
				if self.call('mount -t ext4 %s /mnt/usbssd' % device, shell=True):
					self.log('ssd format: mount failed.')
					raise
		except Exception as e:
			status = 409
			print (e)
			self.exception_log(e, 'ssd_format')
		finally:
			self.service('smbd', 'start')
		return status
